@@ -1,9 +1,11 @@
 from vietocr.optim.optim import ScheduledOptim
+from vietocr.optim.labelsmoothingloss import LabelSmoothingLoss
 from torch.optim import Adam
 from torch import nn
 from vietocr.tool.translate import build_model
 from vietocr.tool.translate import translate
 from vietocr.tool.utils import download_weights
+from vietocr.tool.logger import Logger
 from einops import rearrange
 import yaml
 import torch
@@ -15,7 +17,7 @@ import os
 import matplotlib.pyplot as plt
 
 class Trainer():
-    def __init__(self, config, pretrain=True):
+    def __init__(self, config, pretrained=True):
 
         self.config = config
         self.model, self.vocab = build_model(config)
@@ -31,8 +33,12 @@ class Trainer():
         self.checkpoint = config['trainer']['checkpoint']
         self.export_weights = config['trainer']['export']
         self.metrics = config['trainer']['metrics']
+        logger = config['trainer']['log']
+    
+        if logger:
+            self.logger = Logger(logger) 
 
-        if pretrain:
+        if pretrained:
             download_weights(**config['pretrain'], quiet=config['quiet'])
             self.model.load_state_dict(torch.load(config['pretrain']['cached'], map_location=torch.device(self.device)))
 
@@ -43,7 +49,8 @@ class Trainer():
             Adam(self.model.parameters(), betas=(0.9, 0.98), eps=1e-09),
             0.2, config['transformer']['d_model'], config['optimizer']['n_warmup_steps'])
 
-        self.criterion = nn.CrossEntropyLoss(ignore_index=0) 
+#        self.criterion = nn.CrossEntropyLoss(ignore_index=0) 
+        self.criterion = LabelSmoothingLoss(len(self.vocab), padding_idx=self.vocab.pad, smoothing=0.1)
 
         self.train_gen = DataGen(self.data_root, self.train_annotation, self.vocab, self.device)
         if self.valid_annotation:
@@ -68,16 +75,18 @@ class Trainer():
                     info = 'iter: {:06d} - epoch: {:03d} - train loss: {:.4f}'.format(self.iter, epoch, total_loss/self.print_every)
                     total_loss = 0
                     print(info) 
-                
+                    self.logger.log(info)
+
                 if self.valid_annotation and self.iter % self.valid_every == self.valid_every - 1:
                     val_loss = self.validate()
                     info = 'iter: {:06d} - epoch: {:03d} - val loss: {:.4f}'.format(self.iter, epoch, val_loss)
                     print(info)
-                    
-                    if self.metrics:
-                        acc_full_seq, acc_per_char = self.precision()
-                        info = 'iter: {:06d} - epoch: {:03d} - acc full seq: {:.4f} - acc per char: {:.4f}'.format(self.iter, epoch, acc_full_seq, acc_per_char)
-                        print(info)
+                    self.logger.log(info)
+
+                    acc_full_seq, acc_per_char = self.precision(self.metrics)
+                    info = 'iter: {:06d} - epoch: {:03d} - acc full seq: {:.4f} - acc per char: {:.4f}'.format(self.iter, epoch, acc_full_seq, acc_per_char)
+                    print(info)
+                    self.logger.log(info)
 
                     self.save_checkpoint(self.checkpoint)
                     self.save_weight(self.export_weights)
@@ -128,9 +137,9 @@ class Trainer():
 
         return pred_sents, actual_sents, img_files
 
-    def precision(self):
+    def precision(self, sample=None):
 
-        pred_sents, actual_sents, _ = self.predict(sample=None)
+        pred_sents, actual_sents, _ = self.predict(sample=sample)
 
         acc_full_seq = compute_accuracy(actual_sents, pred_sents, mode='full_sequence')
         acc_per_char = compute_accuracy(actual_sents, pred_sents, mode='per_char')
@@ -185,7 +194,7 @@ class Trainer():
         os.makedirs(path, exist_ok=True)
        
         torch.save(self.model.state_dict(), filename)
-
+    
     def step(self, batch):
         self.model.train()
         
